package net.minecraft.src.railcraft.common.api;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import net.minecraft.src.Block;
import net.minecraft.src.Item;
import net.minecraft.src.ItemStack;

/**
 * A collection of functions that help interface with the different versions of the
 * Buildcraft Liquid API as well as providing a substitute if Buildcraft is not installed.
 *
 * @author CovertJaguar <railcraft.wikispaces.com>
 */
public abstract class LiquidTools
{

    public static final int BUCKET_VOLUME = 1000;

    /**
     * Wrapper function for Buildcraft API to allow Railcraft to work with
     * either Buildcraft API version or none at all.
     * @param bucket An ItemStack representing a filled bucket, can, etc...
     * @return The liquid id for the item.
     */
    public static int getLiquidForBucket(ItemStack bucket)
    {
        if(bucket == null){
            return 0;
        }
        if(bucket.itemID == Item.bucketWater.shiftedIndex) {
            return Block.waterStill.blockID;
        }
        if(bucket.itemID == Item.bucketLava.shiftedIndex) {
            return Block.lavaStill.blockID;
        }
        try {
            Class api = Class.forName("buildcraft.api.API");

            Method[] methods = api.getMethods();
            for(Method m : methods) {
                if(m.getName().equals("getLiquidForBucket")) {
                    return (Integer)m.invoke(null, bucket.itemID);
                }
            }
        } catch (Exception ex) {
        }
        try {
            Class api = Class.forName("buildcraft.api.BuildCraftAPI");

            Method[] methods = api.getMethods();
            for(Method m : methods) {
                if(m.getName().equals("getLiquidForFilledItem")) {
                    return (Integer)m.invoke(null, bucket);
                }
            }
        } catch (Exception ex) {
        }
        return 0;
    }

    /**
     * Helper function to retrieve the list of registered Liquids from the Buildcraft API.
     *
     * Returns an empty list if Buildcraft is not installed.
     *
     * @return A List of LiquidData object, version of LiquidData may vary.
     */
    public static List getLiquidData()
    {
        try {
            Class api = Class.forName("buildcraft.api.BuildCraftAPI");

            if(api == null) {
                api = Class.forName("buildcraft.api.API");
            }

            if(api != null) {
                Field[] fields = api.getFields();
                for(Field f : fields) {
                    if(f.getName().equals("liquids")) {
                        Object obj = f.get(null);
                        if(obj instanceof List) {
                            return (List)obj;
                        }
                    }
                }
            }
        } catch (Exception ex) {
        }
        return new LinkedList();
    }

    /**
     * Helper function to retrieve a map of registered Liquids and their buckets from the Buildcraft API.
     * At the very least it will return water and lava even if Buildcraft is not installed.
     *
     * @return A map of liquidIds and buckets
     */
    public static Map<Integer, ItemStack> getLiquidMap()
    {
        Map<Integer, ItemStack> map = new TreeMap<Integer, ItemStack>();

        for(Object liquidData : getLiquidData()) {
            try {
                Field liquidIdField = liquidData.getClass().getField("liquidId");
                int liquidId = (Integer)liquidIdField.get(liquidData);
                ItemStack bucket = null;
                try {
                    Field filledField = liquidData.getClass().getField("filled");
                    bucket = ((ItemStack)filledField.get(liquidData)).copy();
                } catch (Exception ex) {
                }
                if(bucket == null) {
                    try {
                        Field filledBucketIdField = liquidData.getClass().getField("filledBucketId");
                        int bucketId = (Integer)filledBucketIdField.get(liquidData);
                        bucket = new ItemStack(bucketId, 1, 0);
                    } catch (Exception ex) {
                    }
                }
                if(bucket != null) {
                    map.put(liquidId, bucket);
                }
            } catch (Exception ex) {
            }
        }

        map.put(Block.waterStill.blockID, new ItemStack(Item.bucketWater));
        map.put(Block.lavaStill.blockID, new ItemStack(Item.bucketLava));
        return map;
    }

    /**
     * Wrapper function for Buildcraft API to allow Railcraft to work with
     * either Buildcraft API version or none at all.
     *
     * @param id the id of the liquid in question
     * @return a filled bucket ItemStack
     */
    public static ItemStack getBucketForLiquid(int id)
    {
        if(id == Block.waterStill.blockID) {
            return new ItemStack(Item.bucketWater);
        }
        if(id == Block.lavaStill.blockID) {
            return new ItemStack(Item.bucketLava);
        }
        try {
            List liquids = getLiquidData();

            for(Object lq : liquids) {
                Field liquidId = lq.getClass().getField("liquidId");
                Object li = liquidId.get(lq);
                if(li instanceof Integer) {
                    Integer intId = (Integer)li;
                    if(intId.intValue() == id) {
                        try {
                            Field container = lq.getClass().getField("container");
                            Object c = container.get(lq);
                            if(c instanceof ItemStack) {
                                ItemStack emtpy = (ItemStack)c;
                                if(emtpy.itemID == Item.bucketEmpty.shiftedIndex) {
                                    Field filled = lq.getClass().getField("filled");
                                    Object f = filled.get(lq);
                                    if(f instanceof ItemStack) {
                                        return (ItemStack)f;
                                    }
                                }
                            }
                        } catch (Exception ex) {
                        }

                        try {
                            Field filledBucketId = lq.getClass().getField("filledBucketId");
                            Object fb = filledBucketId.get(lq);
                            if(fb instanceof Integer) {
                                Integer intFB = (Integer)fb;
                                return new ItemStack(intFB, 1, 0);
                            }
                        } catch (Exception ex) {
                        }
                    }
                }
            }
        } catch (Exception ex) {
        }
        return null;
    }
}
